"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const findRoot = require("find-root");
const path_1 = require("path");
const fs_1 = require("fs");
const isAlias = (alias) => (imported) => imported.moduleName.indexOf(alias) === 0;
const hasAlias = (alias) => (imported) => {
    return alias.some((a) => imported.moduleName.indexOf(a) === 0);
};
const createAliasRules = (styleApi, alias) => {
    console.log(alias);
    const { and, hasDefaultMember, hasNamedMembers, hasNamespaceMember, hasNoMember, hasOnlyDefaultMember, hasOnlyNamedMembers, hasOnlyNamespaceMember, isAbsoluteModule, isRelativeModule, member, name, not, startsWithAlphanumeric, startsWithLowerCase, startsWithUpperCase, unicode, } = styleApi;
    return alias.reduce((accumulator, currentValue) => {
        return accumulator.concat([
            // import * as _ from "alias/name";
            { match: and(hasOnlyNamespaceMember, isAbsoluteModule, isAlias(currentValue), not(member(startsWithAlphanumeric))), sort: member(unicode) },
            // import * as Foo from "bar";
            { match: and(hasOnlyNamespaceMember, isAbsoluteModule, isAlias(currentValue), member(startsWithUpperCase)), sort: member(unicode) },
            // import * as foo from "bar";
            { match: and(hasOnlyNamespaceMember, isAbsoluteModule, isAlias(currentValue), member(startsWithLowerCase)), sort: member(unicode) },
            // import _, * as bar from "baz";
            { match: and(hasDefaultMember, hasNamespaceMember, isAbsoluteModule, isAlias(currentValue), not(member(startsWithAlphanumeric))), sort: member(unicode) },
            // import Foo, * as bar from "baz";
            { match: and(hasDefaultMember, hasNamespaceMember, isAbsoluteModule, isAlias(currentValue), member(startsWithUpperCase)), sort: member(unicode) },
            // import foo, * as bar from "baz";
            { match: and(hasDefaultMember, hasNamespaceMember, isAbsoluteModule, isAlias(currentValue), member(startsWithUpperCase)), sort: member(unicode) },
            // import _ from "bar";
            { match: and(hasOnlyDefaultMember, isAbsoluteModule, isAlias(currentValue), not(member(startsWithAlphanumeric))), sort: member(unicode) },
            // import Foo from "bar";
            { match: and(hasOnlyDefaultMember, isAbsoluteModule, isAlias(currentValue), member(startsWithUpperCase)), sort: member(unicode) },
            // import foo from "bar";
            { match: and(hasOnlyDefaultMember, isAbsoluteModule, isAlias(currentValue), member(startsWithLowerCase)), sort: member(unicode) },
            // import _, {bar, …} from "baz";
            { match: and(hasDefaultMember, hasNamedMembers, isAbsoluteModule, isAlias(currentValue), not(member(startsWithAlphanumeric))), sort: member(unicode), sortNamedMembers: name(unicode) },
            // import Foo, {bar, …} from "baz";
            { match: and(hasDefaultMember, hasNamedMembers, isAbsoluteModule, isAlias(currentValue), member(startsWithUpperCase)), sort: member(unicode), sortNamedMembers: name(unicode) },
            // import foo, {bar, …} from "baz";
            { match: and(hasDefaultMember, hasNamedMembers, isAbsoluteModule, isAlias(currentValue), member(startsWithLowerCase)), sort: member(unicode), sortNamedMembers: name(unicode) },
            // import {_, bar, …} from "baz";
            { match: and(hasOnlyNamedMembers, isAbsoluteModule, isAlias(currentValue), not(member(startsWithAlphanumeric))), sort: member(unicode), sortNamedMembers: name(unicode) },
            // import {Foo, bar, …} from "baz";
            { match: and(hasOnlyNamedMembers, isAbsoluteModule, isAlias(currentValue), member(startsWithUpperCase)), sort: member(unicode), sortNamedMembers: name(unicode) },
            // import {foo, bar, …} from "baz";
            { match: and(hasOnlyNamedMembers, isAbsoluteModule, isAlias(currentValue), member(startsWithLowerCase)), sort: member(unicode), sortNamedMembers: name(unicode) },
        ]);
    }, []);
};
const getAlias = (extension, config) => {
    let alias = [];
    Object.keys(config).forEach((key) => {
        if (key.indexOf(extension) >= 0) {
            alias = config[key].custom && config[key].custom.alias;
        }
    });
    return alias || [];
};
function default_1(styleApi, file) {
    const { and, hasDefaultMember, hasNamedMembers, hasNamespaceMember, hasNoMember, hasOnlyDefaultMember, hasOnlyNamedMembers, hasOnlyNamespaceMember, isAbsoluteModule, isRelativeModule, member, name, not, startsWithAlphanumeric, startsWithLowerCase, startsWithUpperCase, unicode, } = styleApi;
    const importSortConfig = JSON.parse(fs_1.readFileSync(`${findRoot(file)}/package.json`, "utf-8")).importSort;
    const alias = getAlias(path_1.extname(file), importSortConfig);
    const customRules = createAliasRules(styleApi, alias);
    return [
        // import "foo"
        { match: and(hasNoMember, isAbsoluteModule) },
        { separator: true },
        // import "./foo"
        { match: and(hasNoMember, isRelativeModule) },
        { separator: true },
        // import * as _ from "bar";
        {
            match: and(hasOnlyNamespaceMember, isAbsoluteModule, not(hasAlias(alias)), not(member(startsWithAlphanumeric))),
            sort: member(unicode),
        },
        // import * as Foo from "bar";
        {
            match: and(hasOnlyNamespaceMember, isAbsoluteModule, not(hasAlias(alias)), member(startsWithUpperCase)),
            sort: member(unicode),
        },
        // import * as foo from "bar";
        {
            match: and(hasOnlyNamespaceMember, isAbsoluteModule, not(hasAlias(alias)), member(startsWithLowerCase)),
            sort: member(unicode),
        },
        // import _, * as bar from "baz";
        {
            match: and(hasDefaultMember, hasNamespaceMember, isAbsoluteModule, not(hasAlias(alias)), not(member(startsWithAlphanumeric))),
            sort: member(unicode),
        },
        // import Foo, * as bar from "baz";
        { match: and(hasDefaultMember, hasNamespaceMember, isAbsoluteModule, not(hasAlias(alias)), member(startsWithUpperCase)), sort: member(unicode) },
        // import foo, * as bar from "baz";
        { match: and(hasDefaultMember, hasNamespaceMember, isAbsoluteModule, not(hasAlias(alias)), member(startsWithUpperCase)), sort: member(unicode) },
        // import _ from "bar";
        { match: and(hasOnlyDefaultMember, isAbsoluteModule, not(hasAlias(alias)), not(member(startsWithAlphanumeric))), sort: member(unicode) },
        // import Foo from "bar";
        { match: and(hasOnlyDefaultMember, isAbsoluteModule, not(hasAlias(alias)), member(startsWithUpperCase)), sort: member(unicode) },
        // import foo from "bar";
        { match: and(hasOnlyDefaultMember, isAbsoluteModule, not(hasAlias(alias)), member(startsWithLowerCase)), sort: member(unicode) },
        // import _, {bar, …} from "baz";
        { match: and(hasDefaultMember, hasNamedMembers, isAbsoluteModule, not(hasAlias(alias)), not(member(startsWithAlphanumeric))), sort: member(unicode), sortNamedMembers: name(unicode) },
        // import Foo, {bar, …} from "baz";
        { match: and(hasDefaultMember, hasNamedMembers, isAbsoluteModule, not(hasAlias(alias)), member(startsWithUpperCase)), sort: member(unicode), sortNamedMembers: name(unicode) },
        // import foo, {bar, …} from "baz";
        { match: and(hasDefaultMember, hasNamedMembers, isAbsoluteModule, not(hasAlias(alias)), member(startsWithLowerCase)), sort: member(unicode), sortNamedMembers: name(unicode) },
        // import {_, bar, …} from "baz";
        { match: and(hasOnlyNamedMembers, isAbsoluteModule, not(hasAlias(alias)), not(member(startsWithAlphanumeric))), sort: member(unicode), sortNamedMembers: name(unicode) },
        // import {Foo, bar, …} from "baz";
        { match: and(hasOnlyNamedMembers, isAbsoluteModule, not(hasAlias(alias)), member(startsWithUpperCase)), sort: member(unicode), sortNamedMembers: name(unicode) },
        // import {foo, bar, …} from "baz";
        { match: and(hasOnlyNamedMembers, isAbsoluteModule, not(hasAlias(alias)), member(startsWithLowerCase)), sort: member(unicode), sortNamedMembers: name(unicode) },
        { separator: true },
        ...customRules,
        { separator: true },
        // import * as _ from "./bar";
        { match: and(hasOnlyNamespaceMember, isRelativeModule, not(member(startsWithAlphanumeric))), sort: member(unicode) },
        // import * as Foo from "./bar";
        { match: and(hasOnlyNamespaceMember, isRelativeModule, member(startsWithUpperCase)), sort: member(unicode) },
        // import * as foo from "./bar";
        { match: and(hasOnlyNamespaceMember, isRelativeModule, member(startsWithLowerCase)), sort: member(unicode) },
        // import _, * as bar from "./baz";
        { match: and(hasDefaultMember, hasNamespaceMember, isRelativeModule, not(member(startsWithAlphanumeric))), sort: member(unicode) },
        // import Foo, * as bar from "./baz";
        { match: and(hasDefaultMember, hasNamespaceMember, isRelativeModule, member(startsWithUpperCase)), sort: member(unicode) },
        // import foo, * as bar from "./baz";
        { match: and(hasDefaultMember, hasNamespaceMember, isRelativeModule, member(startsWithUpperCase)), sort: member(unicode) },
        // import _ from "./bar";
        { match: and(hasOnlyDefaultMember, isRelativeModule, not(member(startsWithAlphanumeric))), sort: member(unicode) },
        // import Foo from "./bar";
        { match: and(hasOnlyDefaultMember, isRelativeModule, member(startsWithUpperCase)), sort: member(unicode) },
        // import foo from "./bar";
        { match: and(hasOnlyDefaultMember, isRelativeModule, member(startsWithLowerCase)), sort: member(unicode) },
        // import _, {bar, …} from "./baz";
        { match: and(hasDefaultMember, hasNamedMembers, isRelativeModule, not(member(startsWithAlphanumeric))), sort: member(unicode), sortNamedMembers: name(unicode) },
        // import Foo, {bar, …} from "./baz";
        { match: and(hasDefaultMember, hasNamedMembers, isRelativeModule, member(startsWithUpperCase)), sort: member(unicode), sortNamedMembers: name(unicode) },
        // import foo, {bar, …} from "./baz";
        { match: and(hasDefaultMember, hasNamedMembers, isRelativeModule, member(startsWithLowerCase)), sort: member(unicode), sortNamedMembers: name(unicode) },
        // import {_, bar, …} from "./baz";
        { match: and(hasOnlyNamedMembers, isRelativeModule, not(member(startsWithAlphanumeric))), sort: member(unicode), sortNamedMembers: name(unicode) },
        // import {Foo, bar, …} from "./baz";
        { match: and(hasOnlyNamedMembers, isRelativeModule, member(startsWithUpperCase)), sort: member(unicode), sortNamedMembers: name(unicode) },
        // import {foo, bar, …} from "./baz";
        { match: and(hasOnlyNamedMembers, isRelativeModule, member(startsWithLowerCase)), sort: member(unicode), sortNamedMembers: name(unicode) },
        { separator: true },
    ];
}
exports.default = default_1;
